[assembly: WebActivatorEx.PreApplicationStartMethod(typeof(AdventureWorksWeb.App_Start.NinjectWebCommon), "Start")]
[assembly: WebActivatorEx.ApplicationShutdownMethodAttribute(typeof(AdventureWorksWeb.App_Start.NinjectWebCommon), "Stop")]

namespace AdventureWorksWeb.App_Start
{
    using System;
    using System.Web;

    using Microsoft.Web.Infrastructure.DynamicModuleHelper;

    using Ninject;
    using Ninject.Web.Common;
    using Models.Entitites;
    using Common;
    using Configuration;
    using Rebus.Ninject;
    using Rebus.Config;
    using Rebus.Routing.TypeBased;
    using Common.Events;
    public static class NinjectWebCommon
    {
        private static readonly Bootstrapper bootstrapper = new Bootstrapper();

        /// <summary>
        /// Starts the application
        /// </summary>
        public static void Start()
        {
            DynamicModuleUtility.RegisterModule(typeof(OnePerRequestHttpModule));
            DynamicModuleUtility.RegisterModule(typeof(NinjectHttpModule));
            bootstrapper.Initialize(CreateKernel);
        }

        /// <summary>
        /// Stops the application.
        /// </summary>
        public static void Stop()
        {
            bootstrapper.ShutDown();
        }

        /// <summary>
        /// Creates the kernel that will manage your application.
        /// </summary>
        /// <returns>The created kernel.</returns>
        private static IKernel CreateKernel()
        {
            var kernel = new StandardKernel();
            try
            {
                kernel.Bind<Func<IKernel>>().ToMethod(ctx => () => new Bootstrapper().Kernel);
                kernel.Bind<IHttpModule>().To<HttpApplicationInitializationHttpModule>();

                RegisterServices(kernel);
                return kernel;
            }
            catch
            {
                kernel.Dispose();
                throw;
            }
        }

        /// <summary>
        /// Load your modules or register your services here!
        /// </summary>
        /// <param name="kernel">The kernel.</param>
        private static void RegisterServices(IKernel kernel)
        {
            kernel.Bind<AdventureWorks>().ToSelf().InRequestScope();
            kernel.Bind<ISearchSettings>().To<AppSettings>().InSingletonScope();

            var settings = kernel.Get<ISearchSettings>();

            var activator = new NinjectContainerAdapter(kernel);

            Configure.With(activator)
                .Transport(t => t.UseAzureServiceBus(settings.BusConnectionString, "AzureSearchDemoWeb"))
                .Routing(r => r.TypeBased().Map<IProductEvent>("products"))
                .Start();
        }
    }
}
